/******************************************************************************
 * (c) Copyright 2002,2003, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile: RepXSLTransformer.java,v $
 * Version:       $Name:  $ $Revision: 1.5 $
 * Last Modified: $Date: 2008/04/30 10:59:44 $
 *****************************************************************************/
package org.netkernel.xml.accessor.xslt;


import org.netkernel.layer0.nkf.INKFLocale;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.util.OrderedCheapMultiMap;
import org.netkernel.layer0.util.RequestScopeClassLoader;
import org.netkernel.xml.util.NKFEntityResolver;
import org.netkernel.xml.util.XMLUtils;
import org.netkernel.xml.xda.*;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.*;

import java.io.IOException;
import java.util.*;

public class RepXSLTransformer implements ErrorListener
{
	private Templates mTemplate;
	private OrderedCheapMultiMap mParameters;
	private String mMime;
	
	/** Creates a new instance of XSLTTransformerAspect */
	protected RepXSLTransformer(INKFRequestContext aContext, IReadableBinaryStreamRepresentation aFrom, Document aFromDoc, URIResolver aResolver, String aSystemId) throws TransformerConfigurationException, IOException, Exception
	{	this(aContext, aFrom, aFromDoc, aResolver, aSystemId,false);
	}
	
	/** Creates a new instance of XSLTTransformerAspect */
	protected RepXSLTransformer(INKFRequestContext aContext,IReadableBinaryStreamRepresentation aFrom, Document aFromDoc, URIResolver aResolver, String aSystemId, boolean aCompiled) throws TransformerConfigurationException, IOException, Exception
	{	
		// get mime
		mMime ="text/xml";
		if(aFromDoc==null)
		{	aFromDoc=XMLUtils.parse(aFrom.getInputStream(), new NKFEntityResolver(aContext));			
		}
		IXDAReadOnly xda=new DOMXDA(aFromDoc,false);
		try
		{	if(xda.isTrue("/*/xsl:output"))
			{	String method=xda.getText("/*/xsl:output/@method",false);
				if(method.equals("html")) mMime="text/html";
				else if(method.equals("text")) mMime="text/plain";
			}
		}
		catch(XPathLocationException e)
		{	//OK we have no output method
		}
		
		// Determine the xsl parameters
		mParameters = new OrderedCheapMultiMap(2);
		IXDAReadOnlyIterator i = xda.readOnlyIterator("/*/xsl:param[@name]");
		while (i.hasNext())
		{	i.next();
			String name = i.getText("@name",false);
			String type= null;
			try
			{	type=i.getText("@nk:class", true);
			}
			catch(XPathLocationException e)
			{	/*no type so default to DOM*/
				type="org.w3c.dom.Document";
			}
			mParameters.put(name, getClassFromSuperStack(type,aContext));
		}
		
		//Build the Xalan transformer
		javax.xml.transform.TransformerFactory tf =aCompiled ?
						(javax.xml.transform.TransformerFactory)org.apache.xalan.xsltc.trax.TransformerFactoryImpl.newInstance() :
                        (javax.xml.transform.TransformerFactory)org.apache.xalan.processor.TransformerFactoryImpl.newInstance( ) ; 
		tf.setURIResolver(aResolver);
		tf.setErrorListener(this);
		
		XMLReader reader=XMLReaderFactory.createXMLReader();
		reader.setEntityResolver(new NKFEntityResolver(aContext));
		reader.setFeature("http://xml.org/sax/features/resolve-dtd-uris", false);
		InputSource src=new InputSource(aFrom.getInputStream());
		src.setSystemId(aSystemId);
		Source source = new SAXSource(reader, src);
		try
		{	mTemplate = tf.newTemplates(source);
		}
		catch(TransformerException e)
		{	e.printStackTrace();
			//Error in SAXSource so retry with stream source to get error metadata - this is still not returning a Source Locator!
			source=new StreamSource(aFrom.getInputStream(), aSystemId);
			tf.setAttribute(org.apache.xalan.processor.TransformerFactoryImpl.FEATURE_SOURCE_LOCATION, true);
			mTemplate=tf.newTemplates(source);
		}
	}
	
	public Templates getTemplate()
	{	return mTemplate;
	}
	
	public OrderedCheapMultiMap getParameters()
	{	return mParameters;
	}
	
	public String getMimeType()
	{	return mMime;
	}
	
	private Class getClassFromSuperStack(String classname, INKFRequestContext aContext) throws Exception
	{	ClassLoader cl=new RequestScopeClassLoader(aContext.getKernelContext().getThisKernelRequest().getRequestScope());  //Use a superstack classloader
		return cl.loadClass(classname);
	}
	
	public void error(TransformerException exception)	throws TransformerException
	{		throw exception;	
	}

	public void fatalError(TransformerException exception)	throws TransformerException
	{	throw exception;	
	}

	public void warning(TransformerException exception) throws TransformerException
	{	throw exception;	
	}

}