/******************************************************************************
  (c) Copyright 2002,2003, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  File:          $RCSfile: DOMXDAParser.java,v $
  Version:       $Name:  $ $Revision: 1.1 $
  Last Modified: $Date: 2008/04/21 09:38:02 $
 *****************************************************************************/
package org.netkernel.xml.transrepresentation;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.module.standard.endpoint.StandardTransreptorImpl;
import org.netkernel.xml.util.XMLUtils;
import org.netkernel.xml.xda.*;
import org.w3c.dom.*;
import org.xml.sax.SAXParseException;

import java.io.*;

/**
 * Parses a binary stream into a DOM
 * @author  tab
 */
public class DOMXDAParser extends StandardTransreptorImpl
{
	public DOMXDAParser()
	{	this.declareThreadSafe();
		this.declareFromRepresentation(IBinaryStreamRepresentation.class);
		this.declareFromRepresentation(String.class);
		this.declareToRepresentation(Document.class);
		this.declareToRepresentation(IXDA.class);
		this.declareName("DOMXDAParser");
		this.declareDescription("parse to DOM or XDA interfaces");
	}
	
	public void onTransrept(INKFRequestContext aContext) throws Exception
	{	
		try
		{
			Object from = aContext.getThisRequest().getPrimary();
			Document d;
			if (from instanceof String)
			{	Reader r = new StringReader((String)from);
				d = XMLUtils.parse(r);
			}
			else if (from instanceof IReadableBinaryStreamRepresentation)
			{	IReadableBinaryStreamRepresentation rbs = (IReadableBinaryStreamRepresentation)from;
				InputStream is = rbs.getInputStream();
				try
				{	d = XMLUtils.parse(is);
				}
				finally
				{	is.close(); 
				}
			}
			else
			{	IBinaryStreamRepresentation input = (IBinaryStreamRepresentation)from;
				ByteArrayOutputStream baos = new ByteArrayOutputStream(1024);
				input.write(baos);
				ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
				d = XMLUtils.parse(bais);
			}
			
			Class requestedRep = aContext.getThisRequest().getRepresentationClass();
			if (requestedRep.isAssignableFrom(Document.class))
			{	aContext.createResponseFrom(d);
			}
			else
			{	DOMXDA domXDA = new DOMXDA(d,false);
				aContext.createResponseFrom(domXDA);
			}
		}
		catch (SAXParseException spe)
		{	aContext.createResponseFrom(spe);
		}
	}
}